package cmd

import (
	"fmt"
	"os"
	"os/signal"
	"samba/pkg/log"
	"samba/pkg/servername"
	"samba/proto"
	"samba/server/hall/handler"
	"samba/util/config"
	"samba/util/model"
	"samba/util/routingKey"
	"samba/util/util"
	"syscall"
	"testing"
)

var token = ""

// 初始化环境
func initEnv() {
	config.Cmd = config.Command{
		LogLevel:   -1,
		ConfigPath: "D:\\work\\samba\\samba\\bin\\config\\hall_dev.xml",
	}
	initLog(&config.Cmd)
	var err = config.Load(config.Cmd.ConfigPath)
	if err != nil {
		log.Error(fmt.Sprintf("load config err: %v", err))
	}
	initRepo()

	handler.InitService()
}

// 释放环境
func releaseEnv() {
	c := make(chan os.Signal, 1)
	signal.Notify(c, syscall.SIGINT, syscall.SIGTERM)
	sig := <-c
	handler.StopService()
	log.Info(fmt.Sprintf("received %s, initiating shutdown...\n", sig))
}

// 测试加载金币
func TestLoadCoins(t *testing.T) {
	initEnv()
	uid1 := int64(1007)
	router := routingKey.DbKey(uid1)
	fmt.Println("router:", router)
	handler.SendMsg(util.Direct(servername.Money), router, token, uid1, proto.ReqGetResourceId, &proto.ReqGetResource{
		UserId:  uid1,
		ResType: model.ResDiamond,
	})
	handler.SendMsg(util.Direct(servername.Money), router, token, uid1, proto.ReqAddResourceId, &proto.ReqAddResource{
		UserId:   uid1,
		ResType:  model.ResDiamond,
		ResValue: 1,
		Reason:   "test",
	})
	releaseEnv()
}

// 测试加载金币
func TestAddClubCoins(t *testing.T) {
	initEnv()
	uid1 := int64(13126370)
	router := routingKey.DbKey(uid1)
	fmt.Println("router:", router)
	handler.SendMsg(util.Direct(servername.Money), router, token, uid1, proto.ReqGetResourceId, &proto.ReqGetResource{
		UserId:  uid1,
		ResType: model.ResClubUserScore,
	})
	handler.SendMsg(util.Direct(servername.Money), router, token, uid1, proto.ReqAddResourceId, &proto.ReqAddResource{
		UserId:   uid1,
		ResType:  model.ResClubUserScore,
		ResValue: 500,
		Reason:   "test",
	})
	releaseEnv()
}

// 清理所有玩家的携带脏数据
func TestCleanTakeCoin(t *testing.T) {
	initEnv()
	model.CleanTakeCoin()
	releaseEnv()
}

// 清理所有玩家的携带脏数据
func TestUserClubInfoOp(t *testing.T) {
	initEnv()
	_, _ = model.NewUserClubInfoOp().Load(12100131, 10000029)
	releaseEnv()
}
