package handler

import (
	"github.com/stretchr/testify/assert"
	"samba/pkg/servername"
	"samba/proto"
	"samba/server/other/code"
	"samba/util/routingKey"
	"samba/util/sender"
	"samba/util/util"
	"testing"
	"time"
)

func Test_onReqGenerateCode(t *testing.T) {
	s := sender.NewMQSender(
		util.Direct(servername.Other),
		routingKey.Other,
	)

	req := &proto.ReqGenerateCode{
		Code:      code.RedeemCodeGenerator{}.UnsafeNext(),
		StartTime: time.Now().Unix(),
		EndTime:   time.Now().Unix() + 3600,
		Count:     2,
		Item:      map[int]int{1000: 100},
		Rule: proto.Rule{
			UserRegStartTime:   0,
			UserRegEndTime:     0,
			UserLastOnlineTime: 0,
			MinVip:             0,
			MinLevel:           0,
			MinPay:             0,
		},
	}
	rsp := &proto.RspGenerateCode{}
	err := s.Call(proto.ReqGenerateCodeId, req, &rsp)

	assert.NoError(t, err)
	assert.Equal(t, rsp.Code, proto.Ok)
}

func Test_onReqUseCode(t *testing.T) {
	s := sender.NewMQSender(
		util.Direct(servername.Other),
		routingKey.Other,
	)
	req := &proto.ReqUseCode{Code: "65XJZN4Q"}
	// 13100590
	err := s.SendWithUId(proto.ReqUseCodeId, 13100590, req)

	assert.NoError(t, err)

}

func Test_onReqCodeUseQuery(t *testing.T) {
	s := sender.NewMQSender(
		util.Direct(servername.Other),
		routingKey.Other,
	)
	req := &proto.ReqCodeUseQuery{
		Limit:  10,
		Offset: 0,
	}

	rsp := &proto.RspCodeUseQuery{}
	err := s.Call(proto.ReqCodeUseQueryId, req, rsp)
	assert.NoError(t, err)
	assert.Equal(t, rsp.Code, proto.Ok)
	for _, use := range rsp.CodeUse {
		t.Logf("%+v", use)
	}

}

func Test_onReqCodeUsePlayerQuery(t *testing.T) {
	s := sender.NewMQSender(
		util.Direct(servername.Other),
		routingKey.Other,
	)
	req := &proto.ReqCodeUsePlayerQuery{
		Code:   "JBXQF9KY",
		Limit:  10,
		Offset: 0,
	}

	rsp := &proto.RspCodeUsePlayerQuery{}
	err := s.Call(proto.ReqCodeUsePlayerQueryId, req, rsp)
	assert.NoError(t, err)
	assert.Equal(t, rsp.Code, proto.Ok)
	for _, use := range rsp.CodeUsePlayer {
		t.Logf("%+v", use)
	}
}
