package handler

import (
	"github.com/rabbitmq/amqp091-go"
	"samba/pkg/log"
	"samba/pkg/xtime"
	"samba/proto"
	"samba/util/util"
)

func onModifyTimeOffset(d *amqp091.Delivery, msg map[string]any) {
	_, _, _, data := ParseMsg(msg)
	req, err := util.MapToStructT[proto.ReqModifyTimeOffset](data)
	if err != nil {
		log.Error(err.Error())
		return
	}
	before := xtime.Now()
	xtime.SetNowTimeOffset(int(req.Offset))
	after := xtime.Now()

	SendMsgToRPC(d, &proto.RspModifyTimeOffset{
		Ts: map[string]proto.TimeResult{
			"other": {
				TimeBeforeStr: before.StdTime().String(),
				TimeAfterStr:  after.StdTime().String(),
			},
		},
	}, "onModifyTimeOffset")

}
