package poker

import (
	"math/rand"
	"sort"
)

var randPaulistaDirtyPoint = []Point{Point3, Point2, PointA, PointK, PointJ, PointQ, Point7, Point6, Point5, Point4}
var randPaulistaDirtyColor = []Color{Spade, Heart, Club, Diamond}

// Paulista脏
type PaulistaDirtyPokers struct {
	allPokers  []*Poker // 洗牌时用
	pokers     []*Poker // 发牌,去掉鬼牌的所有牌
	ghostPoker *Poker
	maxPoint   Point
	sortPokers []*Poker // 判断大牌时使用

	gmPokers     []*Poker // gm配牌
	gmGhostPoker *Poker
	gmMaxPoint   Point
}

func NewPaulistaDirtyPokers() *PaulistaDirtyPokers {
	pokers := &PaulistaDirtyPokers{
		allPokers:  make([]*Poker, 0),
		pokers:     make([]*Poker, 0),
		ghostPoker: nil,
		maxPoint:   0,
	}
	for _, point := range randPaulistaDirtyPoint {
		pokers.allPokers = append(pokers.allPokers, NewPoker(int(Spade), int(point)))
		pokers.allPokers = append(pokers.allPokers, NewPoker(int(Heart), int(point)))
		pokers.allPokers = append(pokers.allPokers, NewPoker(int(Club), int(point)))
		pokers.allPokers = append(pokers.allPokers, NewPoker(int(Diamond), int(point)))
	}
	for _, pk := range pokers.allPokers {
		pokers.sortPokers = append(pokers.sortPokers, pk)
	}
	return pokers
}

func (p *PaulistaDirtyPokers) Pokers() []*Poker {
	return p.pokers
}

func (p *PaulistaDirtyPokers) GhostPoker() *Poker {
	return p.ghostPoker
}

func (p *PaulistaDirtyPokers) MaxPoint() Point {
	return p.maxPoint
}

func (p *PaulistaDirtyPokers) GetColorValue(pk *Poker) int {
	switch pk.Color {
	case Spade:
		return 2
	case Heart:
		return 3
	case Club:
		return 4
	case Diamond:
		return 1
	}
	return 0
}

func (p *PaulistaDirtyPokers) DebugSortPoker() string {
	large := ""
	restPoint := make([]int, 0)
	for _, point := range randPaulistaDirtyPoint {
		if point == p.maxPoint {
			large += NewPoker(int(Diamond), int(point)).ToString() + " "
			large += NewPoker(int(Spade), int(point)).ToString() + " "
			large += NewPoker(int(Heart), int(point)).ToString() + " "
			large += NewPoker(int(Club), int(point)).ToString() + " "
		} else {
			restPoint = append(restPoint, int(point))
		}
	}
	sort.Slice(restPoint, func(i, j int) bool {
		p1 := NewPoker(int(Club), restPoint[i])
		p2 := NewPoker(int(Club), restPoint[j])
		//v1 := p.getPokerValue(p1)
		//v2 := p.getPokerValue(p2)
		//fmt.Printf("p1:%v v1:%v p2:%v v2:%v\n", p1.ToString(), v1, p2.ToString(), v2)
		return p.getPokerValue(p1) < p.getPokerValue(p2)
	})
	small := ""
	for _, point := range restPoint {
		small += Point(point).String() + " "
	}
	return small + "- " + large
}

func (p *PaulistaDirtyPokers) getPokerValue(pk *Poker) int {
	// 暗牌
	if pk.IsDark {
		return -1
	}
	// 最大牌则加花色
	if pk.Point == p.maxPoint {
		return p.GetColorValue(pk)*100 + 14
	} else {
		switch pk.Point {
		case Point3:
			return 13
		case Point2:
			return 12
		case PointA:
			return 11
		case PointK:
			return 10
		case PointJ:
			return 9
		case PointQ:
			return 8
		case Point7:
			return 7
		case Point6:
			return 6
		case Point5:
			return 5
		case Point4:
			return 4
		}
	}
	return 0
}

// Cmp p1<p2:CppSmall
func (p *PaulistaDirtyPokers) Cmp(p1, p2 *Poker) CmpPokerPoint {
	v1 := p.getPokerValue(p1)
	v2 := p.getPokerValue(p2)
	if v1 < v2 {
		return CppSmall
	} else if v1 == v2 {
		return CppEqual
	} else {
		return CppBig
	}
}

// 洗牌
func (p *PaulistaDirtyPokers) Shuffle() {
	for _, pk := range p.pokers {
		pk.IsDark = false
	}
	if !p.gmShuffle() {
		p.randShuffle()
	}
}

func (p *PaulistaDirtyPokers) randShuffle() {
	rand.Shuffle(len(p.allPokers), func(i, j int) {
		p.allPokers[i], p.allPokers[j] = p.allPokers[j], p.allPokers[i]
	})
	pos := int(rand.Int31n(int32(len(randPaulistaDirtyPoint))))
	colorPos := int(rand.Int31n(int32(len(randPaulistaDirtyColor))))
	p.ghostPoker = NewPoker(int(randPaulistaDirtyColor[colorPos]), int(randPaulistaDirtyPoint[pos]))
	maxPos := (pos + len(randPaulistaDirtyPoint) - 1) % len(randPaulistaDirtyPoint)
	p.maxPoint = randPaulistaDirtyPoint[maxPos]
	p.pokers = p.pokers[0:0]
	for _, poker := range p.allPokers {
		if poker.Point == p.ghostPoker.Point && poker.Color == p.ghostPoker.Color {
			continue
		}
		p.pokers = append(p.pokers, poker)
	}

	sort.Slice(p.sortPokers, func(i, j int) bool {
		return p.Cmp(p.sortPokers[i], p.sortPokers[j]) == CppBig
	})
}

func (p *PaulistaDirtyPokers) gmShuffle() bool {
	if len(p.gmPokers) == 0 {
		return false
	}
	p.ghostPoker = p.gmGhostPoker
	p.maxPoint = p.gmMaxPoint
	p.pokers = p.gmPokers

	for _, pk := range p.pokers {
		pk.IsDark = false
	}

	sort.Slice(p.sortPokers, func(i, j int) bool {
		return p.Cmp(p.sortPokers[i], p.sortPokers[j]) == CppBig
	})
	return true
}

func (p *PaulistaDirtyPokers) IsBigPoker(pk *Poker, rank int) bool {
	for pos1, pk1 := range p.sortPokers {
		if pos1 >= rank {
			return false
		}
		if p.Cmp(pk1, pk) == CppEqual {
			return pos1 <= rank
		}
	}
	return false
}

func (p *PaulistaDirtyPokers) IsGoodPoker(pk *Poker) bool {
	// 前两张为好牌，无视花色
	for _, poker := range p.sortPokers[:8] {
		if poker.Point == pk.Point && poker.Color == pk.Color {
			return true
		}
	}
	return false
}

func (p *PaulistaDirtyPokers) IsBadPoker(pk *Poker) bool {
	// 后两张为坏牌，无视花色
	for _, poker := range p.sortPokers[len(p.sortPokers)-8:] {
		if poker.Point == pk.Point && poker.Color == pk.Color {
			return true
		}
	}
	return false
}

func (p *PaulistaDirtyPokers) IsMinPoker(pk *Poker) bool {
	return p.Cmp(pk, p.sortPokers[len(p.sortPokers)-1]) == CppEqual
}

func (p *PaulistaDirtyPokers) SetGmPoker(gmPokers [][]int, ghostPoker int, seat int) {
	if len(gmPokers) == 0 {
		p.gmGhostPoker = nil
		p.gmMaxPoint = 0
		p.gmPokers = nil
		return
	}

	p.gmGhostPoker = NewPoker(ghostPoker)
	pos := 0
	for i, point := range randPaulistaDirtyPoint {
		if p.gmGhostPoker.Point == point {
			pos = i
		}
	}
	maxPos := (pos + len(randPaulistaDirtyPoint) - 1) % len(randPaulistaDirtyPoint)
	p.gmMaxPoint = randPaulistaDirtyPoint[maxPos]
	p.gmPokers = make([]*Poker, 0)

	sets := make([]*Poker, 0) // 补集
	for _, pk := range p.allPokers {
		exist := false
		for _, gmPks := range gmPokers {
			for _, pkInt := range gmPks {
				if pk.ToInt() == pkInt {
					exist = true
				}
			}
		}
		if !exist {
			sets = append(sets, pk)
		}
	}

	step := 0
	for i := 0; i < len(gmPokers); i++ {
		pokers := gmPokers[(len(gmPokers)-seat+i)%len(gmPokers)]
		pks := make([]*Poker, 0)
		for _, pok := range pokers {
			pk := NewPoker(pok)
			if len(pks) >= 3 {
				break
			}
			if pk.Point == p.gmGhostPoker.Point && pk.Color == p.gmGhostPoker.Color {
				pks = append(pks, sets[step])
				step++
			} else {
				pks = append(pks, pk)
			}
		}
		if len(pks) < 3 {
			for i := len(pks); i < 3; i++ {
				pks = append(pks, sets[step])
				step++
			}
		}
		p.gmPokers = append(p.gmPokers, pks...)
	}
	for ; step < len(sets); step++ {
		p.gmPokers = append(p.gmPokers, sets[step])
	}
}
