package util

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/go-redis/redis/v8"
)

type resultT[T any] struct {
	Value T
	Err   error
}

func MapToStructT[T any](m interface{}) (*T, error) {
	var result = resultT[T]{Err: nil}
	var data []byte
	data, result.Err = json.Marshal(m)
	if result.Err != nil {
		return &result.Value, result.Err
	}
	result.Err = json.Unmarshal(data, &result.Value)
	return &result.Value, result.Err
}

func MapToStruct(m interface{}, s interface{}) error {
	jsonData, err := json.Marshal(m)
	if err != nil {
		return err
	}
	return json.Unmarshal(jsonData, &s)
}

func StructToMap(data interface{}) (map[string]interface{}, error) {
	jsonData, err := json.Marshal(data)
	if err != nil {
		return nil, err
	}
	s := make(map[string]interface{})
	err = json.Unmarshal(jsonData, &s)
	return s, err
}

func Struct2Redis(rdb *redis.Client, key string, s interface{}) error {
	mapValues, err := Serialize(s)
	if err != nil {
		return err
	}
	return rdb.HSet(context.Background(), key, mapValues).Err()
}

func Redis2Struct[T any](rdb *redis.Client, key string) (*T, error) {
	mapValues, err := rdb.HGetAll(context.Background(), key).Result()
	if err != nil {
		return nil, err
	}
	if len(mapValues) == 0 {
		return nil, fmt.Errorf("key:%v does not exist", key)
	}
	ret := resultT[T]{}
	ret.Err = Deserialize(mapValues, &ret.Value)
	return &ret.Value, ret.Err
}
