package util

import (
	"math/rand/v2"
	"time"
)

// RandSecond 随机秒数
// 随机精度在ms间
// start ,end 都是秒
func RandSecond(start, end int) time.Duration {
	start *= 1000 // ms
	end *= 1000   // ms

	r := RandRange(start, end)

	return time.Duration(r) * time.Millisecond
}

// RandRange 随机取范围内的值
func RandRange[T ~int | ~int8 | ~int16 | ~int32 | ~int64 | ~uint | ~uint8 | ~uint16 | ~uint32 | ~uint64](min, max T) T {
	minI64 := int64(min)
	maxI64 := int64(max)
	res := rand.Int64N(maxI64-minI64+1) + minI64
	return T(res)
}

// RandHappened 随机发生概率
// 给定一个概率 (0~100)，返回是否发生
func RandHappened(probability int) (happened bool) {
	if probability <= 0 {
		return false
	}
	if probability >= 100 {
		return true
	}
	return RandRange(0, 100) < probability
}

// RandChoice 随机选择切片中的一个元素
func RandChoice[T ~[]E, E any](s T) E {
	return s[RandRange(0, len(s)-1)]
}
